"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const path_1 = __importDefault(require("path"));
const kleur_1 = __importDefault(require("kleur"));
const file_check_1 = require("./checks/file-check");
const command_check_1 = require("./checks/command-check");
const match_check_1 = require("./checks/match-check");
const search_check_1 = require("./checks/search-check");
const console_1 = require("./util/console");
const fs_1 = require("./util/fs");
const string_1 = require("./util/string");
const report_1 = require("./report/report");
function prettyPath(pathToPrint) {
    return path_1.default.isAbsolute(pathToPrint)
        ? pathToPrint
            .replace(process.cwd() + path_1.default.sep, '.' + path_1.default.sep)
            .replace(process.cwd(), '.' + path_1.default.sep)
            .replace(/\\/g, '/')
            .replace(/^\.\//, '')
        : pathToPrint.replace(/\\/g, '/').replace(/^\.\//, '');
}
async function main(checksFileArg, targetDirectoryArgs, useSubfolders, cleanResultsDirectory) {
    const checksFile = await findChecksFile(checksFileArg);
    const checks = getChecks(checksFile);
    console_1.println(kleur_1.default.blue(`running checks from ${prettyPath(checksFile)}`));
    console_1.println();
    const targetDirectories = await resolveTargetDirectories(targetDirectoryArgs, useSubfolders);
    const results = [];
    const resultsDirectory = path_1.default.join(process.cwd(), 'autocheck-reports');
    if (cleanResultsDirectory) {
        fs_1.cleanDirectoryContent(resultsDirectory);
    }
    await report_1.copySupportingFiles(resultsDirectory);
    let i = 1;
    for (let directory of targetDirectories) {
        console_1.println(kleur_1.default.blue(`checking directory (${i++}/${targetDirectories.length}): ${prettyPath(directory)}`));
        const completedChecks = new Map();
        const checkResults = [];
        let j = 1;
        for (const check of checks) {
            if (check.ignore) {
                continue;
            }
            console_1.println(`  running check (${j++}/${checks.length}): `, check.label);
            let checkResult;
            if (check.if) {
                const dependency = completedChecks.get(check.if);
                if (dependency) {
                    if (dependency.status === 'passed') {
                        checkResult = await performCheck(check, directory, completedChecks);
                    }
                    else {
                        checkResult = {
                            config: check,
                            status: 'skipped',
                            error: `Check skipped because the check it depends on, ${string_1.quote(check.if)}, ${dependency.status === 'skipped' ? 'was skipped' : 'failed'}.`,
                        };
                        console_1.println(kleur_1.default.gray(`    check skipped because ${string_1.quote(check.if)} ${dependency.status === 'skipped' ? 'was skipped' : 'failed'}.`));
                    }
                }
                else {
                    checkResult = {
                        config: check,
                        status: 'skipped',
                        error: `Check skipped because the check it depends on, ${string_1.quote(check.if)}, was not found. Make sure the referenced check exists and appears before this check.`,
                    };
                    console_1.println(kleur_1.default.gray(`    check skipped because ${string_1.quote(check.if)} was not found`));
                }
            }
            else {
                checkResult = await performCheck(check, directory, completedChecks);
            }
            if (checkResult) {
                completedChecks.set(check.label, {
                    status: checkResult.status,
                    output: checkResult.output,
                });
                checkResults.push(checkResult);
                if (checkResult.status === 'passed') {
                    console_1.println(kleur_1.default.green('    check ' + checkResult.status));
                }
                else if (checkResult.status === 'failed') {
                    console_1.print(kleur_1.default.red('    check failed'));
                    console_1.println(checkResult.error ? ': ' + checkResult.error : '');
                }
            }
            else {
                console_1.println(kleur_1.default.gray('    check has no result'));
            }
        }
        const result = {
            title: path_1.default.basename(directory),
            checks: checkResults,
            fileContents: fs_1.getFileCache(directory),
        };
        results.push(result);
        const reportFilePath = await report_1.createReport(result, resultsDirectory);
        console_1.println('  generated report: ', prettyPath(reportFilePath));
        console_1.println();
    }
    if (results.length > 1) {
        console_1.println(kleur_1.default.blue('generating per-check reports...'));
        const perCheckResultsDirectory = await generatePerCheckReports(results, resultsDirectory, cleanResultsDirectory);
        console_1.println();
        console_1.println(kleur_1.default.blue('generating indexes...'));
        let outputFile = await report_1.generateReportsIndex(resultsDirectory, 'Autocheck Reports');
        console_1.println('  per-target index: ', prettyPath(outputFile));
        outputFile = await report_1.generateReportsIndex(perCheckResultsDirectory, 'Autocheck Per-check Reports');
        console_1.println('  per-check index: ', prettyPath(outputFile));
        console_1.println();
    }
    console_1.println(kleur_1.default.blue('finishing up...'));
}
exports.main = main;
async function resolveTargetDirectories(targetDirectoryArgs, useSubfolders) {
    const targetDirectories = [];
    for (let arg of targetDirectoryArgs) {
        arg = arg.replace(/"/g, '');
        const directory = path_1.default.isAbsolute(arg) ? arg : path_1.default.join(process.cwd(), arg);
        if (await fs_1.directoryExists(directory)) {
            if (useSubfolders) {
                const directories = await fs_1.listDirectories(directory, directory, {
                    recursive: false,
                });
                targetDirectories.push(...directories);
            }
            else {
                targetDirectories.push(directory);
            }
        }
        else {
            console.log('target directory not found, skipping: ', directory);
        }
    }
    return targetDirectories;
}
async function findChecksFile(checksFilePath) {
    const resolvedChecksFilePath = path_1.default.isAbsolute(checksFilePath)
        ? checksFilePath
        : path_1.default.join(process.cwd(), checksFilePath);
    if (!(await fs_1.fileExists(resolvedChecksFilePath))) {
        console.log('checks file not found:', resolvedChecksFilePath);
        process.exit();
    }
    return resolvedChecksFilePath;
}
function getChecks(filePath) {
    try {
        return require(filePath);
    }
    catch (err) {
        console.log('unable to read checks file:', err);
        process.exit();
    }
}
async function performCheck(checkConfiguration, targetDirectory, context) {
    switch (checkConfiguration.type) {
        case 'file':
            return file_check_1.performFileCheck(checkConfiguration, targetDirectory);
        case 'command':
            return command_check_1.performCommandCheck(checkConfiguration, targetDirectory);
        case 'match':
            return match_check_1.performMatchCheck(checkConfiguration, targetDirectory, context);
        case 'search':
            return search_check_1.performSearchCheck(checkConfiguration, targetDirectory);
    }
}
/**
 * Turns the list of per-target results to a list of per-check results,
 * in the same format that the frontend expects, just changing titles,
 * labels, and file paths.
 */
async function generatePerCheckReports(results, resultsDirectory, cleanResultsDirectory) {
    const perCheckResults = new Map();
    results.forEach((result) => {
        result.checks.forEach((check) => {
            const transformedResult = perCheckResults.get(check.config.label) ?? {
                title: check.config.label,
                fileContents: {},
                checks: [],
            };
            Object.entries(result.fileContents).forEach(([key, value]) => {
                transformedResult.fileContents[result.title + '/' + key] = value;
            });
            const transformedCheck = Object.assign({}, check, {
                config: Object.assign({}, check.config, {
                    label: result.title,
                }),
                results: check.results?.map((file) => {
                    return Object.assign({}, file, {
                        relativePath: file.relativePath ? `${result.title}/${file.relativePath}` : undefined,
                    });
                }),
            });
            transformedResult.checks.push(transformedCheck);
            perCheckResults.set(check.config.label, transformedResult);
        });
    });
    const perCheckResultsDirectory = path_1.default.join(resultsDirectory, 'per-check');
    if (cleanResultsDirectory) {
        fs_1.cleanDirectoryContent(resultsDirectory);
    }
    await report_1.copySupportingFiles(perCheckResultsDirectory);
    let i = 1;
    for (const check of perCheckResults.values()) {
        const outputFile = await report_1.createReport(check, perCheckResultsDirectory, String(i++).padStart(2, '0') + '. ');
        console_1.println('  check report: ', prettyPath(outputFile));
    }
    return perCheckResultsDirectory;
}
