"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.performCommandCheck = void 0;
const path_1 = __importDefault(require("path"));
const upath_1 = __importDefault(require("upath"));
const execa_1 = __importDefault(require("execa"));
const fs_1 = require("../util/fs");
async function performCommandCheck(checkConfiguration, targetDirectory) {
    if (!checkConfiguration.command || checkConfiguration.command.trim().length === 0) {
        return {
            config: checkConfiguration,
            status: 'failed',
            error: 'No `command` specified for this check.',
        };
    }
    if (checkConfiguration.runInCygwin &&
        (!checkConfiguration.cygwinBin || checkConfiguration.cygwinBin.trim().length === 0)) {
        return {
            config: checkConfiguration,
            status: 'failed',
            error: 'The `cygwinBin` option is required to run a command in Cygwin.',
        };
    }
    const directory = checkConfiguration.directory
        ? resolvePath(checkConfiguration.directory, targetDirectory)
        : targetDirectory;
    if (directory && !(await fs_1.directoryExists(directory))) {
        return {
            config: checkConfiguration,
            result: { directory },
            status: 'failed',
            error: `Command working directory not found or inaccessible: ${directory}`,
        };
    }
    const command = checkConfiguration.command;
    const commandParts = checkConfiguration.runInCygwin
        ? makeCygwinCommand(command, directory, checkConfiguration.cygwinBin)
        : command.split(' ').map((part) => part.trim());
    let inputStream;
    if (checkConfiguration.input) {
        const input = resolvePath(checkConfiguration.input, targetDirectory);
        if (input && (await fs_1.fileExists(input))) {
            inputStream = fs_1.createReadStream(input, { encoding: 'utf-8' });
        }
    }
    let output;
    let error;
    let exitCode;
    try {
        const subprocess = execa_1.default(commandParts[0], commandParts.slice(1), {
            shell: true,
            localDir: directory,
            cwd: directory,
            input: inputStream,
            all: true,
        });
        const cancelTimeout = killAfterTimeout(subprocess, 10000);
        const commandResult = await subprocess;
        cancelTimeout();
        output = commandResult.all;
        exitCode = commandResult.exitCode;
    }
    catch (err) {
        output = err.all;
        exitCode = err.exitCode ?? -1;
        if (err.isCanceled) {
            error =
                'command cancelled after timeout. if the command expects std input, set the `input` option.';
        }
    }
    return {
        config: checkConfiguration,
        status: exitCode === 0 ? 'passed' : 'failed',
        result: {
            directory: checkConfiguration.runInCygwin ? toCygpath(directory) : directory,
            exitCode,
        },
        output,
        error,
    };
}
exports.performCommandCheck = performCommandCheck;
function resolvePath(pathToResolve, parentDirectory) {
    return path_1.default.isAbsolute(pathToResolve) ? pathToResolve : path_1.default.join(parentDirectory, pathToResolve);
}
function makeCygwinCommand(command, directory, cygwinBin) {
    const bashExecutable = path_1.default.join(cygwinBin, 'bash.exe');
    const bashArgs = ['--norc', '--noprofile', '-l'];
    const updatePath = 'export PATH=/usr/local/bin:/usr/bin:$PATH';
    const changeDirectory = `cd \\"${toCygpath(directory)}\\"`;
    // TODO: escape `command` for use in the double quotes
    // (so the user doesn't have to do that manually)
    const bashCommand = ['-c', `"${updatePath} && ${changeDirectory} && ${command}"`];
    return [bashExecutable, ...bashArgs, ...bashCommand];
}
function toCygpath(location) {
    const locationPosix = upath_1.default.toUnix(location);
    const root = path_1.default.parse(locationPosix).root;
    const rootLetter = root.replace(':/', '/');
    return locationPosix.replace(root, `/cygdrive/${rootLetter.toLowerCase()}`);
}
function killAfterTimeout(subprocess, timeout) {
    const timeoutId = setTimeout(() => {
        subprocess.cancel();
    }, timeout);
    return () => {
        clearTimeout(timeoutId);
    };
}
