"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.performMatchCheck = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = require("../util/fs");
const string_1 = require("../util/string");
const trailingSpacesRegex = /[ \t]+([\r\n]+)/g;
async function performMatchCheck(checkConfiguration, targetDirectory, context) {
    if (!checkConfiguration.expected) {
        return {
            config: checkConfiguration,
            status: 'failed',
            error: 'No `expected` value specified for this check.',
        };
    }
    if (!checkConfiguration.actual) {
        return {
            config: checkConfiguration,
            status: 'failed',
            error: 'No `actual` value specified for this check.',
        };
    }
    try {
        const expected = await resolveExpression(checkConfiguration.expected, targetDirectory, context);
        const actual = await resolveExpression(checkConfiguration.actual, targetDirectory, context);
        let expectedTrimmed = expected.trim();
        let actualTrimmed = actual.trim();
        if (checkConfiguration.ignoreTrailingSpaces ?? true) {
            expectedTrimmed = expectedTrimmed.replace(trailingSpacesRegex, (_match, group1) => group1);
            actualTrimmed = actualTrimmed.replace(trailingSpacesRegex, (_match, group1) => group1);
        }
        const passed = expectedTrimmed === actualTrimmed;
        return {
            config: checkConfiguration,
            status: passed ? 'passed' : 'failed',
            expected: expectedTrimmed,
            actual: actualTrimmed,
        };
    }
    catch (err) {
        return {
            config: checkConfiguration,
            status: 'failed',
            error: err,
        };
    }
}
exports.performMatchCheck = performMatchCheck;
async function resolveExpression(value, targetDirectory, context) {
    if (value.startsWith('output:')) {
        const referenceName = value.replace('output:', '');
        const reference = context.get(referenceName);
        if (reference) {
            if (reference.output) {
                return reference.output;
            }
            else {
                throw `The check ${string_1.quote(referenceName)} has no output.`;
            }
        }
        else {
            throw `Unable to read output of check, the check ${string_1.quote(referenceName)} was not found.`;
        }
    }
    if (value.startsWith('file:')) {
        const fileName = value.replace('file:', '');
        const filePath = path_1.default.isAbsolute(fileName) ? fileName : path_1.default.join(targetDirectory, fileName);
        if (await fs_1.fileExists(filePath)) {
            try {
                return await fs_1.readString(filePath);
            }
            catch {
                throw `Unable to read file: ${filePath}`;
            }
        }
        else {
            throw `Unable to read file: ${filePath}`;
        }
    }
    return value;
}
