"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isInRange = exports.findCommentRanges = exports.search = exports.getSearchMatches = exports.configPatternsToRegex = exports.performSearchCheck = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = require("../util/fs");
const regex_1 = require("../util/regex");
async function performSearchCheck(checkConfiguration, targetDirectory) {
    const results = [];
    const matchingFiles = await fs_1.expandGlobs(targetDirectory, checkConfiguration.filePatterns);
    const patternRegexes = configPatternsToRegex(checkConfiguration.patterns, {
        matchAsRegex: checkConfiguration.matchAsRegex,
        matchCase: checkConfiguration.matchCase,
    });
    for (const file of matchingFiles) {
        // Fallback to the basename for files without an extension (e.g. makefiles)
        const extension = path_1.default.extname(file.path).slice(1) || path_1.default.basename(file.path);
        const result = {
            type: extension.toLowerCase(),
            relativePath: file.relativePath,
            path: file.path,
            matches: [],
        };
        if (fs_1.isTextFile(result.type)) {
            const fileContent = await fs_1.readStringAndCache(file.path, file.relativePath, targetDirectory);
            result.matches = getSearchMatches(fileContent, patternRegexes, checkConfiguration.skipComments);
        }
        if (result.matches.length > 0) {
            results.push(result);
        }
    }
    let status;
    let error;
    if ((checkConfiguration.passWhen ?? 'found') === 'found') {
        status = results.length > 0 ? 'passed' : 'failed';
        error = results.length > 0 ? undefined : 'No match found';
    }
    else {
        status = results.length === 0 ? 'passed' : 'failed';
        error = results.length === 0 ? undefined : 'Unexpected match found';
    }
    return {
        config: checkConfiguration,
        status,
        results,
        error,
    };
}
exports.performSearchCheck = performSearchCheck;
function configPatternsToRegex(patterns, options) {
    const patternRegexes = [];
    for (const pattern of patterns) {
        const regex = regex_1.patternToRegex(pattern, {
            isRegex: options.matchAsRegex ?? false,
            matchCase: options.matchCase ?? false,
        });
        if (regex) {
            patternRegexes.push(regex);
        }
        else {
            console.log('unable to convert pattern to regex, skipping invalid pattern: ', pattern);
        }
    }
    return patternRegexes;
}
exports.configPatternsToRegex = configPatternsToRegex;
function getSearchMatches(fileContent, patterns, skipComments) {
    let matches = search(fileContent, patterns);
    if (skipComments) {
        const delimiters = Array.isArray(skipComments) ? skipComments : [['/*', '*/'], '//'];
        const commentRanges = findCommentRanges(fileContent, delimiters);
        if (commentRanges.length > 0) {
            matches = matches.filter((match) => {
                return isInRange(commentRanges, match.offsetInFile, match.text.length) === false;
            });
        }
    }
    return matches.sort((a, b) => a.offsetInFile - b.offsetInFile);
}
exports.getSearchMatches = getSearchMatches;
// The number of characters available to display a result in the list of
// results. It's a rough estimate, since the display font is not mono space.
const SEARCH_RESULT_SPACE = 40;
function search(fileContent, patterns) {
    const matches = [];
    for (const regex of patterns) {
        Array.from(fileContent.matchAll(regex)).forEach((match) => {
            const result = {
                text: match[0],
                context: ['', ''],
                line: 0,
                offset: 0,
                offsetInFile: 0,
                lineSpan: 1,
            };
            if (match.index !== undefined) {
                const matchLength = result.text.length;
                // Add the result position
                const startPosition = offsetToPosition(match.index, fileContent);
                result.line = startPosition.line;
                result.offset = startPosition.offset;
                result.offsetInFile = startPosition.offsetInFile;
                // Add the total number of lines
                const endPosition = offsetToPosition(match.index + matchLength - 1, fileContent);
                result.lineSpan = endPosition.line - result.line + 1;
                // Add the result context
                if (matchLength < SEARCH_RESULT_SPACE) {
                    const halfContextSpace = Math.floor((SEARCH_RESULT_SPACE - matchLength) / 2);
                    // Read up to half the available context space before the match
                    let contextBefore = readUntilNewline('backward', fileContent, match.index - 1, halfContextSpace);
                    // Read up to half the available context space after the match
                    let contextAfter = readUntilNewline('forward', fileContent, match.index + matchLength, halfContextSpace);
                    // If the context before is less than half the available space, use the remaining space
                    // for the context after
                    if (contextBefore.length < halfContextSpace) {
                        contextAfter = readUntilNewline('forward', fileContent, match.index + matchLength, halfContextSpace + (halfContextSpace - contextBefore.length));
                    }
                    // If the context after is less than half the available space, use the remaining space
                    // for the context before
                    else if (contextAfter.length < halfContextSpace) {
                        contextBefore = readUntilNewline('backward', fileContent, match.index - 1, halfContextSpace + (halfContextSpace - contextAfter.length));
                    }
                    result.context[0] = contextBefore;
                    result.context[1] = contextAfter;
                }
            }
            matches.push(result);
        });
    }
    return matches;
}
exports.search = search;
function findCommentRanges(text, commentDelimiters) {
    const commentRanges = [];
    for (const delimiter of commentDelimiters) {
        const regex = Array.isArray(delimiter)
            ? regex_1.makeMatchedDelimiterRegex(delimiter[0], delimiter[1])
            : regex_1.makeSingleDelimiterRegex(delimiter);
        const matchedRanges = Array.from(text.matchAll(regex))
            .filter((match) => match[0] !== undefined && match[0] !== null)
            .map((match) => {
            return [match.index, match[0].length];
        })
            .filter((range) => {
            return isInRange(commentRanges, range[0], range[1]) === false;
        });
        commentRanges.push(...matchedRanges);
    }
    return commentRanges;
}
exports.findCommentRanges = findCommentRanges;
function isInRange(existingRanges, index, length) {
    return existingRanges.some(([rangeIndex, rangeLength]) => {
        return index >= rangeIndex && index < rangeIndex + rangeLength && length <= rangeLength;
    });
}
exports.isInRange = isInRange;
const newLineRegex = /(\r\n)|(\n)/g;
function offsetToPosition(offset, text) {
    const position = {
        offsetInFile: offset,
        line: 0,
        offset: 0,
    };
    const newLinesBeforeOffset = (text.slice(0, offset).match(newLineRegex) ?? []).length;
    position.line = newLinesBeforeOffset + 1; // +1 since lines are 1-indexed
    if (offset > 0) {
        for (let i = offset - 1; i > -1; i--) {
            if (text[i] === '\n') {
                break;
            }
            position.offset++;
        }
    }
    return position;
}
function readUntilNewline(direction, text, startIndex, maxLength) {
    let readChars = '';
    let i = startIndex;
    while (true) {
        if (i < 0 || i >= text.length || readChars.length === maxLength) {
            break;
        }
        if (text[i] === '\n' || (text[i] === '\r' && text[i + 1] === '\n')) {
            break;
        }
        if (direction === 'forward') {
            readChars = readChars + text[i];
            i++;
        }
        else {
            readChars = text[i] + readChars;
            i--;
        }
    }
    return readChars;
}
