"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTextFile = exports.directoryExists = exports.fileExists = exports.copyDirectory = exports.cleanDirectoryContent = exports.expandGlobs = exports.listDirectories = exports.listFiles = exports.writeString = exports.getFileCache = exports.readStringAndCache = exports.readString = exports.createReadStream = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const picomatch_1 = __importDefault(require("picomatch"));
const upath_1 = __importDefault(require("upath"));
const rimraf_1 = __importDefault(require("rimraf"));
const copy_dir_1 = __importDefault(require("copy-dir"));
exports.createReadStream = fs_1.default.createReadStream;
async function readString(filePath) {
    return fs_1.default.promises.readFile(filePath, 'utf8');
}
exports.readString = readString;
const fileCache = new Map();
async function readStringAndCache(filePath, key, namespace) {
    const content = await fs_1.default.promises.readFile(filePath, 'utf8');
    const namespaceContent = fileCache.get(namespace) ?? new Map();
    namespaceContent.set(key, content);
    fileCache.set(namespace, namespaceContent);
    return content;
}
exports.readStringAndCache = readStringAndCache;
function getFileCache(namespace) {
    const cache = fileCache.get(namespace) ?? new Map();
    const output = {};
    cache.forEach((value, key) => {
        output[key] = value;
    });
    return output;
}
exports.getFileCache = getFileCache;
async function writeString(filePath, string) {
    return fs_1.default.promises.writeFile(filePath, string);
}
exports.writeString = writeString;
async function listFiles(currentDirectory, rootDirectory) {
    const files = [];
    const names = await fs_1.default.promises.readdir(currentDirectory);
    for (const file of names) {
        const filePath = path_1.default.join(currentDirectory, file);
        const stat = await fs_1.default.promises.stat(filePath);
        if (stat.isDirectory()) {
            files.push(...(await listFiles(filePath, rootDirectory)));
        }
        else {
            files.push({
                path: filePath,
                relativePath: upath_1.default
                    .toUnix(filePath)
                    .replace(upath_1.default.toUnix(path_1.default.join(rootDirectory, path_1.default.sep)), ''),
            });
        }
    }
    return files;
}
exports.listFiles = listFiles;
async function listDirectories(currentDirectory, rootDirectory, options = { recursive: true }) {
    const directories = [];
    const names = await fs_1.default.promises.readdir(currentDirectory);
    for (const name of names) {
        const fullPath = path_1.default.join(currentDirectory, name);
        const stat = await fs_1.default.promises.stat(fullPath);
        if (stat.isDirectory()) {
            directories.push(fullPath);
            if (options.recursive) {
                directories.push(...(await listDirectories(fullPath, rootDirectory, options)));
            }
        }
    }
    return directories;
}
exports.listDirectories = listDirectories;
async function expandGlobs(rootDirectory, globs) {
    const files = await listFiles(rootDirectory, rootDirectory);
    const isMatch = picomatch_1.default(globs, { nocase: true });
    return files.filter((file) => isMatch(file.relativePath));
}
exports.expandGlobs = expandGlobs;
function cleanDirectoryContent(directory) {
    return new Promise((resolve, reject) => {
        rimraf_1.default(path_1.default.join(directory, '*'), (err) => {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
exports.cleanDirectoryContent = cleanDirectoryContent;
function copyDirectory(source, destination, options) {
    return new Promise((resolve, reject) => {
        copy_dir_1.default(source, destination, options, (err) => {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
exports.copyDirectory = copyDirectory;
async function fileExists(filePath) {
    try {
        return (await fs_1.default.promises.stat(filePath)).isFile();
    }
    catch {
        return false;
    }
}
exports.fileExists = fileExists;
async function directoryExists(directoryPath) {
    try {
        return (await fs_1.default.promises.stat(directoryPath)).isDirectory();
    }
    catch {
        return false;
    }
}
exports.directoryExists = directoryExists;
function isTextFile(extension) {
    return [
        'txt',
        'md',
        'c',
        'cpp',
        'cs',
        'h',
        'hpp',
        'makefile',
        'java',
        'py',
        'html',
        'css',
        'js',
        'json',
        'xml',
        'svg',
    ].includes(extension);
}
exports.isTextFile = isTextFile;
